@@ -17,12 +17,15 @@
 #include "aqhbci_l.h"
 #include "hbci_l.h"
 #include "user_l.h"
+#include "account_l.h"
 #include <aqhbci/provider.h>
 #include <aqbanking/job_be.h>
+#include <aqbanking/account_be.h>
 #include <gwenhywfar/debug.h>
 #include <gwenhywfar/misc.h>
 #include <gwenhywfar/gui.h>
 #include <gwenhywfar/text.h>
+#include <gwenhywfar/stringlist.h>
 
 #include <stdlib.h>
 #include <assert.h>
@@ -36,10 +39,12 @@ GWEN_INHERIT_FUNCTIONS(AH_JOB);
 
 
 
+
+
 AH_JOB *AH_Job_new(const char *name,
 		   AB_USER *u,
-		   const char *accountId,
-		   int jobVersion) {
+                   AB_ACCOUNT *acc,
+                   int jobVersion) {
   AH_JOB *j;
   GWEN_XMLNODE *node;
   GWEN_XMLNODE *jobNode=0;
@@ -65,8 +70,6 @@ AH_JOB *AH_Job_new(const char *name,
   GWEN_LIST_INIT(AH_JOB, j);
   GWEN_INHERIT_INIT(AH_JOB, j);
   j->name=strdup(name);
-  if (accountId)
-    j->accountId=strdup(accountId);
   j->user=u;
   j->signers=GWEN_StringList_new();
   j->log=GWEN_StringList_new();
@@ -257,39 +260,41 @@ AH_JOB *AH_Job_new(const char *name,
   } /* if paramName */
 
   /* get UPD jobs (if any) */
-  if (accountId) {
+  if (acc) {
     GWEN_DB_NODE *updgroup;
+    GWEN_DB_NODE *updnode=NULL;
 
-    updgroup=AH_User_GetUpd(u);
-    assert(updgroup);
-    updgroup=GWEN_DB_GetGroup(updgroup, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
-                              accountId);
+    updgroup=AH_User_GetUpdForAccount(u, acc);
     if (updgroup) {
       const char *code;
 
       code=GWEN_XMLNode_GetProperty(jobNode, "code", 0);
       if (code) {
-        GWEN_DB_NODE *n;
-
         DBG_NOTICE(AQHBCI_LOGDOMAIN, "Code is \"%s\"", code);
-        n=GWEN_DB_GetFirstGroup(updgroup);
-        while(n) {
-          if (strcasecmp(GWEN_DB_GetCharValue(n, "job", 0, ""),
+        updnode=GWEN_DB_GetFirstGroup(updgroup);
+        while(updnode) {
+          if (strcasecmp(GWEN_DB_GetCharValue(updnode, "job", 0, ""),
                          code)==0) {
-            GWEN_DB_NODE *dgr;
-
-            /* upd job found */
-            dgr=GWEN_DB_GetGroup(j->jobParams,
-                                 GWEN_DB_FLAGS_OVERWRITE_GROUPS,
-                                 "upd");
-            assert(dgr);
-            GWEN_DB_AddGroupChildren(dgr, n);
             break;
           }
-          n=GWEN_DB_GetNextGroup(n);
+          updnode=GWEN_DB_GetNextGroup(updnode);
         } /* while */
       } /* if code given */
     } /* if updgroup for the given account found */
+    if (updnode) {
+      GWEN_DB_NODE *dgr;
+
+      /* upd job found */
+      dgr=GWEN_DB_GetGroup(j->jobParams, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "upd");
+      assert(dgr);
+      GWEN_DB_AddGroupChildren(dgr, updnode);
+    }
+    else if (needsBPD) {
+      DBG_INFO(AQHBCI_LOGDOMAIN,"Job \"%s\" not enabled for account \"%u\"",
+               name, AB_Account_GetUniqueId(acc));
+      AH_Job_free(j);
+      return NULL;
+    }
   } /* if accountId given */
 
   /* sample flags from XML file */
@@ -385,6 +390,33 @@ AH_JOB *AH_Job_new(const char *name,
   j->msgResults=AH_Result_List_new();
   j->messages=AB_Message_List_new();
 
+  /* check BPD for job specific SEPA descriptors */
+  if (needsBPD) {
+    GWEN_DB_NODE *dbT;
+
+    dbT=GWEN_DB_FindFirstGroup(j->jobParams, "SupportedSepaFormats");
+    if (dbT) {
+      GWEN_STRINGLIST *descriptors;
+      unsigned int i;
+      const char *s;
+
+      descriptors=GWEN_StringList_new();
+      while(dbT) {
+        for (i=0; i<10; i++) {
+          s=GWEN_DB_GetCharValue(dbT, "format", i, NULL);
+          if (!(s && *s))
+            break;
+          GWEN_StringList_AppendString(descriptors, s, 0, 1);
+        }
+        dbT=GWEN_DB_FindNextGroup(dbT, "SupportedSepaFormats");
+      }
+      if (GWEN_StringList_Count(descriptors)>0)
+        j->sepaDescriptors=descriptors;
+      else
+        GWEN_StringList_free(descriptors);
+    }
+  }
+
   AH_Job_Log(j, GWEN_LoggerLevel_Info,
              "HBCI-Job created");
 
@@ -401,10 +433,10 @@ void AH_Job_free(AH_JOB *j) {
       GWEN_StringList_free(j->challengeParams);
       GWEN_StringList_free(j->log);
       GWEN_StringList_free(j->signers);
+      GWEN_StringList_free(j->sepaDescriptors);
       free(j->responseName);
       free(j->code);
       free(j->name);
-      free(j->accountId);
       free(j->dialogId);
       free(j->expectedSigner);
       free(j->expectedCrypter);
@@ -413,6 +445,7 @@ void AH_Job_free(AH_JOB *j) {
       GWEN_DB_Group_free(j->jobParams);
       GWEN_DB_Group_free(j->jobArguments);
       GWEN_DB_Group_free(j->jobResponses);
+      GWEN_DB_Group_free(j->sepaProfile);
       AH_Result_List_free(j->msgResults);
       AH_Result_List_free(j->segResults);
       AB_Message_List_free(j->messages);
@@ -1043,14 +1076,6 @@ const char *AH_Job_StatusName(AH_JOB_STATUS st) {
 
 
 
-const char *AH_Job_GetAccountId(const AH_JOB *j) {
-  assert(j);
-  assert(j->usage);
-  return j->accountId;
-}
-
-
-
 int AH_Job_Process(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx){
 
   assert(j);
@@ -1423,578 +1448,6 @@ void AH_Job_Dump(const AH_JOB *j, FILE *f, unsigned int insert) {
 
 
 
-int AH_Job__CommitSystemData(AH_JOB *j, int doLock) {
-  GWEN_DB_NODE *dbCurr;
-  AB_USER *u;
-  AB_BANKING *ab;
-  AH_HBCI *h;
-  const char *p;
-  int i;
-//  int modBank;
-//  int modCust;
-  GWEN_MSGENGINE *e;
-  int bpdDeleted;
-
-  DBG_NOTICE(AQHBCI_LOGDOMAIN, "Committing data");
-  assert(j);
-  assert(j->usage);
-
-//  modBank=0;
-//  bpdDeleted=0;
-
-  u=j->user;
-  assert(u);
-  h=AH_Job_GetHbci(j);
-  assert(h);
-  ab=AH_Job_GetBankingApi(j);
-  assert(ab);
-  e=AH_User_GetMsgEngine(j->user);
-  assert(e);
-
-  dbCurr=GWEN_DB_GetFirstGroup(j->jobResponses);
-  while(dbCurr) {
-    GWEN_DB_NODE *dbRd;
-
-    dbRd=GWEN_DB_GetGroup(dbCurr, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "data");
-    if (dbRd) {
-      dbRd=GWEN_DB_GetFirstGroup(dbRd);
-    }
-    if (dbRd) {
-      DBG_NOTICE(AQHBCI_LOGDOMAIN,
-                 "Checking group \"%s\"", GWEN_DB_GroupName(dbRd));
-      if (strcasecmp(GWEN_DB_GroupName(dbRd), "bpd")==0){
-        AH_BPD *bpd;
-        GWEN_DB_NODE *n;
-
-        DBG_NOTICE(AQHBCI_LOGDOMAIN, "Found BPD");
-        bpd=AH_User_GetBpd(j->user);
-
-        AH_Bpd_SetBpdVersion(bpd,
-                             GWEN_DB_GetIntValue(dbRd,
-                                                 "version",
-                                                 0, 0));
-
-        /* read bank name */
-        p=GWEN_DB_GetCharValue(dbRd, "name", 0, 0);
-        if (p)
-          AH_Bpd_SetBankName(bpd, p);
-        AH_Bpd_SetJobTypesPerMsg(bpd,
-                                 GWEN_DB_GetIntValue(dbRd,
-                                                     "jobtypespermsg",
-                                                     0, 0));
-        AH_Bpd_SetMaxMsgSize(bpd,
-                             GWEN_DB_GetIntValue(dbRd,
-                                                 "maxmsgsize",
-                                                 0, 0));
-        /* read languages */
-        n=GWEN_DB_GetGroup(dbRd,
-                           GWEN_PATH_FLAGS_NAMEMUSTEXIST,
-                           "languages");
-        if (n) {
-          AH_Bpd_ClearLanguages(bpd);
-          for (i=0;;i++) {
-            int k;
-
-            k=GWEN_DB_GetIntValue(n, "language", i, 0);
-            if (k) {
-              if (AH_Bpd_AddLanguage(bpd, k)) {
-                DBG_ERROR(AQHBCI_LOGDOMAIN, "Too many languages (%d)", i);
-                break;
-              }
-            }
-            else
-              break;
-          }
-        } /* for */
-
-        /* read supported version */
-        n=GWEN_DB_GetGroup(dbRd,
-                           GWEN_PATH_FLAGS_NAMEMUSTEXIST,
-                           "versions");
-        if (n) {
-          AH_Bpd_ClearHbciVersions(bpd);
-          for (i=0;;i++) {
-            int k;
-
-            k=GWEN_DB_GetIntValue(n, "version", i, 0);
-            if (k) {
-              if (AH_Bpd_AddHbciVersion(bpd, k)) {
-                DBG_ERROR(AQHBCI_LOGDOMAIN, "Too many versions (%d)", i);
-                break;
-              }
-            }
-            else
-              break;
-          }
-        } /* for */
-//        modCust=1;
-        /* FIXME: remove this AH_Bank_SetBpd(b, AH_Bpd_dup(bpd)); */
-//	modBank=1;
-	if (!bpdDeleted) {
-	  AH_Bpd_ClearBpdJobs(bpd);
-	  AH_Bpd_ClearAddr(bpd);
-          bpdDeleted=1;
-	}
-      } /* if BPD found */
-
-      else if (strcasecmp(GWEN_DB_GroupName(dbRd), "ComData")==0){
-	/* communication parameters */
-	GWEN_DB_NODE *currService;
-	AH_BPD *bpd;
-
-	DBG_INFO(AQHBCI_LOGDOMAIN, "Found communication infos");
-        bpd=AH_User_GetBpd(j->user);
-	assert(bpd);
-
-	if (!bpdDeleted) {
-	  AH_Bpd_ClearBpdJobs(bpd);
-          AH_Bpd_ClearAddr(bpd);
-	  bpdDeleted=1;
-	}
-
-	currService=GWEN_DB_FindFirstGroup(dbRd, "service");
-	while(currService) {
-	  AH_BPD_ADDR *ba;
-
-	  ba=AH_BpdAddr_FromDb(currService);
-	  if (ba) {
-            DBG_INFO(AQHBCI_LOGDOMAIN, "Adding service");
-	    AH_Bpd_AddAddr(bpd, ba);
-	  }
-	  currService=GWEN_DB_FindNextGroup(currService, "service");
-	}
-
-//        modCust=1;
-      } /* if ComData found */
-
-      else if (strcasecmp(GWEN_DB_GroupName(dbRd), "PinTanBPD")==0){
-        /* special extension of BPD for PIN/TAN mode */
-        GWEN_DB_NODE *bn;
-        GWEN_DB_NODE *currJob;
-        AH_BPD *bpd;
-
-        bpd=AH_User_GetBpd(j->user);
-        assert(bpd);
-	if (!bpdDeleted) {
-	  AH_Bpd_ClearBpdJobs(bpd);
-          AH_Bpd_ClearAddr(bpd);
-	  bpdDeleted=1;
-	}
-        bn=AH_Bpd_GetBpdJobs(bpd, GWEN_MsgEngine_GetProtocolVersion(e));
-        assert(bn);
-
-        currJob=GWEN_DB_FindFirstGroup(dbRd, "job");
-        while(currJob) {
-          const char *jobName;
-          int needTAN;
-          GWEN_DB_NODE *dbJob;
-
-          jobName=GWEN_DB_GetCharValue(currJob, "job", 0, 0);
-          assert(jobName);
-          dbJob=GWEN_DB_GetGroup(bn, GWEN_DB_FLAGS_DEFAULT,
-                                 jobName);
-          assert(dbJob);
-          needTAN=strcasecmp(GWEN_DB_GetCharValue(currJob,
-                                                  "needTan",
-                                                  0,
-                                                  "N"),
-                             "J")==0;
-          GWEN_DB_SetIntValue(dbJob, GWEN_DB_FLAGS_OVERWRITE_VARS,
-                              "needTan", needTAN);
-          currJob=GWEN_DB_FindNextGroup(currJob, "job");
-        } /* while */
-//        modCust=1;
-      } /* if PIN/TAN extension found */
-
-      else if (strcasecmp(GWEN_DB_GroupName(dbRd), "SegResult")==0){
-        GWEN_DB_NODE *dbRes;
-
-        dbRes=GWEN_DB_GetFirstGroup(dbRd);
-        while(dbRes) {
-          if (strcasecmp(GWEN_DB_GroupName(dbRes), "result")==0) {
-            int code;
-//            const char *text;
-  
-            code=GWEN_DB_GetIntValue(dbRes, "resultcode", 0, 0);
-//            text=GWEN_DB_GetCharValue(dbRes, "text", 0, 0);
-            if (code==3920) {
-              int i;
-
-              AH_User_ClearTanMethodList(u);
-              for (i=0; ; i++) {
-                int j;
-
-                j=GWEN_DB_GetIntValue(dbRes, "param", i, 0);
-		if (j==0)
-		  break;
-		DBG_NOTICE(AQHBCI_LOGDOMAIN, "Adding allowed TAN method %d", j);
-		AH_User_AddTanMethod(u, j);
-	      } /* for */
-	      if (i==0) {
-                /* add single step if empty list */
-		DBG_INFO(AQHBCI_LOGDOMAIN, "No allowed TAN method reported, assuming 999");
-		AH_User_AddTanMethod(u, 999);
-	      }
-	    }
-          } /* if result */
-          dbRes=GWEN_DB_GetNextGroup(dbRes);
-        } /* while */
-      }
-      else if (strcasecmp(GWEN_DB_GroupName(dbRd), "GetKeyResponse")==0){
-	/* TODO: Read the key received and ask the user to accept it */
-      }
-
-      else if (strcasecmp(GWEN_DB_GroupName(dbRd), "SecurityMethods")==0){
-        GWEN_DB_NODE *dbT;
-
-        dbT=GWEN_DB_FindFirstGroup(dbRd, "SecProfile");
-	while(dbT) {
-	  const char *code;
-          int version;
-  
-	  code=GWEN_DB_GetCharValue(dbT, "code", 0, NULL);
-	  version=GWEN_DB_GetIntValue(dbT, "version", 0, -1);
-	  if (code && (version>0)) {
-	    DBG_ERROR(AQHBCI_LOGDOMAIN,
-		      "Bank supports mode %s %d",
-		      code, version);
-	    /* TODO: store possible modes */
-	  }
-	  dbT=GWEN_DB_FindNextGroup(dbT, "SecProfile");
-	} /* while */
-      }
-
-      else {
-        GWEN_XMLNODE *bpdn;
-        int segver;
-        /* check for BPD job */
-
-        DBG_INFO(AQHBCI_LOGDOMAIN, "Checking whether \"%s\" is a BPD job",
-                 GWEN_DB_GroupName(dbRd));
-        segver=GWEN_DB_GetIntValue(dbRd, "head/version", 0, 0);
-        /* get segment description (first try id, then code) */
-        bpdn=GWEN_MsgEngine_FindNodeByProperty(e,
-                                               "SEG",
-                                               "id",
-                                               segver,
-                                               GWEN_DB_GroupName(dbRd));
-        if (!bpdn)
-          bpdn=GWEN_MsgEngine_FindNodeByProperty(e,
-                                                 "SEG",
-                                                 "code",
-                                                 segver,
-                                                 GWEN_DB_GroupName(dbRd));
-        if (bpdn) {
-          DBG_DEBUG(AQHBCI_LOGDOMAIN, "Found a candidate");
-          if (atoi(GWEN_XMLNode_GetProperty(bpdn, "isbpdjob", "0"))) {
-            /* segment contains a BPD job, move contents */
-            GWEN_DB_NODE *bn;
-            AH_BPD *bpd;
-            char numbuffer[32];
-
-            DBG_NOTICE(AQHBCI_LOGDOMAIN, "Found BPD job \"%s\"",
-                       GWEN_DB_GroupName(dbRd));
-            bpd=AH_User_GetBpd(j->user);
-            assert(bpd);
-	    if (!bpdDeleted) {
-	      AH_Bpd_ClearBpdJobs(bpd);
-	      AH_Bpd_ClearAddr(bpd);
-	      bpdDeleted=1;
-	    }
-            bn=AH_Bpd_GetBpdJobs(bpd, GWEN_MsgEngine_GetProtocolVersion(e));
-            assert(bn);
-            bn=GWEN_DB_GetGroup(bn, GWEN_DB_FLAGS_DEFAULT,
-                                GWEN_DB_GroupName(dbRd));
-            assert(bn);
-
-            if (GWEN_Text_NumToString(segver,
-                                      numbuffer,
-                                      sizeof(numbuffer)-1,
-                                      0)<1) {
-              DBG_NOTICE(AQHBCI_LOGDOMAIN, "Buffer too small");
-              abort();
-            }
-            bn=GWEN_DB_GetGroup(bn, GWEN_DB_FLAGS_OVERWRITE_GROUPS,
-                                numbuffer);
-            assert(bn);
-
-            GWEN_DB_AddGroupChildren(bn, dbRd);
-            /* remove "head" and "segment" group */
-            GWEN_DB_DeleteGroup(bn, "head");
-	    GWEN_DB_DeleteGroup(bn, "segment");
-	    DBG_INFO(AQHBCI_LOGDOMAIN, "Added BPD Job %s:%d",
-                     GWEN_DB_GroupName(dbRd), segver);
-//            modCust=1;
-          } /* if isbpdjob */
-          else {
-	    DBG_INFO(AQHBCI_LOGDOMAIN,
-		     "Segment \"%s\" is known but not as a BPD job",
-		     GWEN_DB_GroupName(dbRd));
-	  }
-        } /* if segment found */
-        else {
-          DBG_WARN(AQHBCI_LOGDOMAIN, "Did not find segment \"%s\" (%d) ignoring",
-                   GWEN_DB_GroupName(dbRd), segver);
-        }
-      }
-
-      if (strcasecmp(GWEN_DB_GroupName(dbRd), "UserData")==0){
-        /* UserData found */
-        DBG_NOTICE(AQHBCI_LOGDOMAIN, "Found UserData");
-        AH_User_SetUpdVersion(j->user,
-			      GWEN_DB_GetIntValue(dbRd,
-						  "version",
-						  0, 0));
-      }
-      else if (strcasecmp(GWEN_DB_GroupName(dbRd), "AccountData")==0 ||
-	       strcasecmp(GWEN_DB_GroupName(dbRd), "AccountData2")==0){
-	const char *accountId;
-        const char *accountSuffix;
-        const char *userName;
-        const char *accountName;
-        const char *bankCode;
-	const char *custId;
-        const char *iban;
-        AB_ACCOUNT *acc;
-        GWEN_DB_NODE *gr;
-        AH_BPD *bpd;
-        GWEN_DB_NODE *dbUpd;
-	int accCreated;
-	int mayModifyAcc=1;
-        int rv;
-
-        DBG_INFO(AQHBCI_LOGDOMAIN, "Found AccountData");
-        AH_Job_Log(j, GWEN_LoggerLevel_Info,
-                   "Job contains account data");
-
-        /* account data found */
-	accountId=GWEN_DB_GetCharValue(dbRd, "accountId", 0, 0);
-	if (accountId==NULL)
-	  accountId=I18N("AH_JOB|-- no account id --");
-	accountSuffix=GWEN_DB_GetCharValue(dbRd, "accountSubId", 0, 0);
-        accountName=GWEN_DB_GetCharValue(dbRd, "account/name", 0, 0);
-        userName=GWEN_DB_GetCharValue(dbRd, "name1", 0, 0);
-	iban=GWEN_DB_GetCharValue(dbRd, "iban", 0, 0);
-	bankCode=GWEN_DB_GetCharValue(dbRd, "bankCode", 0, 0);
-	if (bankCode==NULL)
-	  bankCode=I18N("AH_JOB|-- no bank code --");
-        assert(bankCode);
-        custId=GWEN_DB_GetCharValue(dbRd, "customer", 0, 0);
-        assert(custId);
-	if (1) {
-          GWEN_BUFFER *mbuf;
-
-          mbuf=GWEN_Buffer_new(0, 128, 0, 1);
-          GWEN_Buffer_AppendString(mbuf, I18N("Received account:"));
-	  GWEN_Buffer_AppendString(mbuf, " ");
-          GWEN_Buffer_AppendString(mbuf, bankCode);
-          GWEN_Buffer_AppendString(mbuf, " / ");
-          if (accountName)
-            GWEN_Buffer_AppendString(mbuf, accountName);
-          else
-            GWEN_Buffer_AppendString(mbuf, accountId);
-	  if (accountSuffix) {
-	    GWEN_Buffer_AppendString(mbuf, "(");
-	    GWEN_Buffer_AppendString(mbuf, accountSuffix);
-	    GWEN_Buffer_AppendString(mbuf, ")");
-	  }
-	  GWEN_Gui_ProgressLog(0,
-			       GWEN_LoggerLevel_Notice,
-			       GWEN_Buffer_GetStart(mbuf));
-	  GWEN_Buffer_free(mbuf);
-        }
-
-        acc=AB_Banking_FindAccount(ab, AH_PROVIDER_NAME,
-                                   "de", /* TODO: get country */
-                                   bankCode,
-                                   accountId,
-                                   accountSuffix);
-	if (acc) {
-          DBG_NOTICE(AQHBCI_LOGDOMAIN,
-                     "Account \"%s\" already exists",
-                     accountId);
-	  accCreated=0;
-	  rv=AB_Banking_BeginExclUseAccount(ab, acc);
-	  if (rv<0) {
-	    DBG_ERROR(AQHBCI_LOGDOMAIN, "here (%d)", rv);
-	    mayModifyAcc=0;
-	  }
-	  else
-	    mayModifyAcc=1;
-	}
-        else {
-          DBG_NOTICE(AQHBCI_LOGDOMAIN, "Creating account \"%s\"", accountId);
-	  accCreated=1;
-	  mayModifyAcc=1;
-          acc=AB_Banking_CreateAccount(ab, AH_PROVIDER_NAME);
-          assert(acc);
-          AB_Account_SetCountry(acc, "de");
-          AB_Account_SetBankCode(acc, bankCode);
-	  AB_Account_SetAccountNumber(acc, accountId);
-          DBG_NOTICE(AQHBCI_LOGDOMAIN,
-                     "Setting user \"%s\" for account \"%s\"",
-                     AB_User_GetUserId(u),
-                     accountId);
-          AB_Account_SetUser(acc, j->user);
-          AB_Account_SetSelectedUser(acc, j->user);
-        }
-
-	if (mayModifyAcc) {
-	  /* modify account */
-	  if (accountName)
-	    AB_Account_SetAccountName(acc, accountName);
-	  if (userName)
-	    AB_Account_SetOwnerName(acc, userName);
-	  if (iban)
-	    AB_Account_SetIBAN(acc, iban);
-          if (accountSuffix && *accountSuffix)
-            AB_Account_SetSubAccountId(acc, accountSuffix);
-
-	  if (strcasecmp(GWEN_DB_GroupName(dbRd), "AccountData2")==0) {
-	    /* KTV in version 2 available */
-	    AH_Account_AddFlags(acc, AH_BANK_FLAGS_KTV2);
-	    DBG_NOTICE(AQHBCI_LOGDOMAIN, "Extended account information is available");
-	    GWEN_Gui_ProgressLog(0,
-				 GWEN_LoggerLevel_Notice,
-				 I18N("Extended account information available"));
-	  }
-	  else {
-	    AH_Account_SubFlags(acc, AH_BANK_FLAGS_KTV2);
-	    DBG_WARN(AQHBCI_LOGDOMAIN, "Extended account information is not available, some jobs might not work");
-	    GWEN_Gui_ProgressLog(0,
-				 GWEN_LoggerLevel_Warning,
-				 I18N("Extended account information is not available, some jobs might not work"));
-	  }
-
-	  /* set bank name */
-	  bpd=AH_User_GetBpd(j->user);
-	  if (bpd) {
-	    const char *s;
-
-	    s=AH_Bpd_GetBankName(bpd);
-	    if (s)
-	      AB_Account_SetBankName(acc, s);
-	  }
-
-	  /* get UPD jobs */
-	  dbUpd=AH_User_GetUpd(j->user);
-	  assert(dbUpd);
-	  dbUpd=GWEN_DB_GetGroup(dbUpd, GWEN_DB_FLAGS_OVERWRITE_GROUPS,
-				 accountId);
-	  assert(dbUpd);
-	  gr=GWEN_DB_GetFirstGroup(dbRd);
-	  while(gr) {
-	    if (strcasecmp(GWEN_DB_GroupName(gr), "updjob")==0) {
-	      /* found an upd job */
-	      DBG_NOTICE(AQHBCI_LOGDOMAIN, "Adding UPD job");
-	      GWEN_DB_AddGroup(dbUpd, GWEN_DB_Group_dup(gr));
-	    }
-	    gr=GWEN_DB_GetNextGroup(gr);
-	  } /* while */
-	}
-
-        if (accCreated)
-	  AB_Banking_AddAccount(ab, acc);
-	else {
-	  if (mayModifyAcc) {
-	    rv=AB_Banking_EndExclUseAccount(ab, acc, 0);
-	    if (rv<0) {
-	      DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
-	      AB_Banking_EndExclUseAccount(ab, acc, 1); /* abort */
-	    }
-	  }
-	}
-//        modCust=1;
-      } /* if accountData */
-
-      if (strcasecmp(GWEN_DB_GroupName(dbRd), "BankMsg")==0){
-        const char *subject;
-        const char *text;
-
-        DBG_NOTICE(AQHBCI_LOGDOMAIN, "Found a bank message");
-	GWEN_Gui_ProgressLog(0,
-			     GWEN_LoggerLevel_Notice,
-			     I18N("Bank message received"));
-        subject=GWEN_DB_GetCharValue(dbRd, "subject", 0, "(Kein Betreff)");
-        text=GWEN_DB_GetCharValue(dbRd, "text", 0, 0);
-        if (subject && text) {
-	  AB_MESSAGE *amsg;
-          GWEN_TIME *ti;
-
-          ti=GWEN_CurrentTime();
-	  amsg=AB_Message_new();
-	  AB_Message_SetSubject(amsg, subject);
-	  AB_Message_SetText(amsg, text);
-	  AB_Message_SetDateReceived(amsg, ti);
-	  GWEN_Time_free(ti);
-	  AB_Message_SetUserId(amsg, AB_User_GetUniqueId(u));
-	  AB_Message_List_Add(amsg, j->messages);
-
-	  if (1) {
-	    GWEN_DB_NODE *dbTmp;
-
-            /* save message, later this will no longer be necessary */
-	    dbTmp=GWEN_DB_Group_new("bank message");
-	    GWEN_DB_SetCharValue(dbTmp, GWEN_DB_FLAGS_OVERWRITE_VARS,
-				 "subject", subject);
-	    GWEN_DB_SetCharValue(dbTmp, GWEN_DB_FLAGS_OVERWRITE_VARS,
-				 "text", text);
-	    if (AH_HBCI_SaveMessage(h, j->user, dbTmp)) {
-	      DBG_ERROR(AQHBCI_LOGDOMAIN, "Could not save this message:");
-	      GWEN_DB_Dump(dbTmp, 2);
-	    }
-	    GWEN_DB_Group_free(dbTmp);
-	  }
-
-        } /* if subject and text given */
-      } /* if bank msg */
-
-
-    } /* if response data found */
-    dbCurr=GWEN_DB_GetNextGroup(dbCurr);
-  } /* while */
-
-  DBG_NOTICE(AQHBCI_LOGDOMAIN, "Finished.");
-  return 0;
-}
-
-
-
-int AH_Job_CommitSystemData(AH_JOB *j, int doLock) {
-  AB_USER *u;
-  AB_BANKING *ab;
-  int rv, rv2;
-
-  u=j->user;
-  assert(u);
-
-  ab=AH_Job_GetBankingApi(j);
-  assert(ab);
-
-  /* lock user */
-  if (doLock) {
-    rv=AB_Banking_BeginExclUseUser(ab, u);
-    if (rv<0) {
-      DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
-      return rv;
-    }
-  }
-
-  /* commit data */
-  rv2=AH_Job__CommitSystemData(j, doLock);
-
-  if (doLock) {
-    /* unlock user */
-    rv=AB_Banking_EndExclUseUser(ab, u, 0);
-    if (rv<0) {
-      DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
-      AB_Banking_EndExclUseUser(ab, u, 1); /* abandon */
-      return rv;
-    }
-  }
-
-  return rv2;
-}
 
 
 
@@ -2551,6 +2004,172 @@ void AH_Job_AddTransfer(AH_JOB *j, AB_TRANSACTION *t) {
 
 
 
+static int AH_Job__SepaProfileSupported(GWEN_DB_NODE *profile,
+                                        const GWEN_STRINGLIST *descriptors) {
+  GWEN_STRINGLISTENTRY *se;
+  char pattern[13];
+  const char *s;
+
+  /* patterns shall always have the form *xxx.yyy.zz* */
+  pattern[0]=pattern[11]='*';
+  pattern[12]='\0';
+
+  /* Well formed type strings are exactly 10 characters long. Others
+   * will either not match or be rejected by the exporter. */
+  strncpy(pattern+1, GWEN_DB_GetCharValue(profile, "type", 0, ""), 10);
+  se=GWEN_StringList_FirstEntry(descriptors);
+  while(se) {
+    s=GWEN_StringListEntry_Data(se);
+    if (s && GWEN_Text_ComparePattern(s, pattern, 1)!=-1) {
+      /* record the descriptor matching this profile */
+      GWEN_DB_SetCharValue(profile, GWEN_DB_FLAGS_OVERWRITE_VARS,
+                           "descriptor", s);
+      break;
+    }
+    se=GWEN_StringListEntry_Next(se);
+  }
+  if (se)
+    return 1;
+  else
+    return 0;
+}
 
 
 
+static int AH_Job__SortSepaProfiles(const void *a, const void *b) {
+  GWEN_DB_NODE **ppa=(GWEN_DB_NODE **)a;
+  GWEN_DB_NODE **ppb=(GWEN_DB_NODE **)b;
+  GWEN_DB_NODE *pa=*ppa;
+  GWEN_DB_NODE *pb=*ppb;
+  int res;
+
+  /* This function is supposed to return a list of profiles in order
+   * of decreasing precedence. */
+  res=strcmp(GWEN_DB_GetCharValue(pb, "type", 0, ""),
+             GWEN_DB_GetCharValue(pa, "type", 0, ""));
+  if (res)
+    return res;
+
+  res=strcmp(GWEN_DB_GetCharValue(pb, "name", 0, ""),
+             GWEN_DB_GetCharValue(pa, "name", 0, ""));
+
+  return res;
+}
+
+
+
+GWEN_DB_NODE *AH_Job_FindSepaProfile(AH_JOB *j, const char *type,
+                                     const char *name) {
+  const GWEN_STRINGLIST *descriptors;
+  GWEN_DB_NODE *dbProfiles;
+
+  assert(j);
+
+  if (j->sepaDescriptors)
+    descriptors=j->sepaDescriptors;
+  else
+    descriptors=AH_User_GetSepaDescriptors(j->user);
+  if (GWEN_StringList_Count(descriptors)==0) {
+    DBG_ERROR(AQHBCI_LOGDOMAIN,
+              "No SEPA descriptor found, please update your account information");
+    return NULL;
+  }
+
+  if (name) {
+    GWEN_DB_NODE *profile;
+
+    profile=AB_Banking_GetImExporterProfile(AH_Job_GetBankingApi(j),
+                                            "sepa", name);
+    if (!profile) {
+      DBG_ERROR(AQHBCI_LOGDOMAIN,
+                "Profile \"%s\" not available", name);
+      return NULL;
+    }
+    if (GWEN_Text_ComparePattern(GWEN_DB_GetCharValue(profile, "type", 0, ""),
+                                 type, 1)==-1) {
+      DBG_ERROR(AQHBCI_LOGDOMAIN,
+                "Profile \"%s\" does not match type speecification (\"%s\")",
+                name, type);
+      return NULL;
+    }
+    if (!AH_Job__SepaProfileSupported(profile, descriptors)) {
+      DBG_ERROR(AQHBCI_LOGDOMAIN,
+                "Profile \"%s\" not supported by bank server", name);
+      return NULL;
+    }
+    j->sepaProfile=profile;
+    return profile;
+  }
+
+  if (!type)
+    return j->sepaProfile;
+
+  if (j->sepaProfile) {
+    if (GWEN_Text_ComparePattern(GWEN_DB_GetCharValue(j->sepaProfile, "type",
+                                                      0, ""),
+                                 type, 1)!=-1)
+      return j->sepaProfile;
+    else {
+      GWEN_DB_Group_free(j->sepaProfile);
+      j->sepaProfile=NULL;
+    }
+  }
+
+  dbProfiles=AB_Banking_GetImExporterProfiles(AH_Job_GetBankingApi(j), "sepa");
+  if (dbProfiles) {
+    GWEN_DB_NODE *n, *nn;
+    unsigned int pCount=0;
+
+    n=GWEN_DB_GetFirstGroup(dbProfiles);
+    while(n) {
+      nn=n;
+      n=GWEN_DB_GetNextGroup(n);
+
+      if (GWEN_Text_ComparePattern(GWEN_DB_GetCharValue(nn, "type", 0, ""),
+                                   type, 1)!=-1 &&
+          AH_Job__SepaProfileSupported(nn, descriptors))
+        pCount++;
+      else {
+        GWEN_DB_UnlinkGroup(nn);
+        GWEN_DB_Group_free(nn);
+      }
+    }
+
+    if (pCount) {
+      GWEN_DB_NODE **orderedProfiles;
+      unsigned int i;
+
+      orderedProfiles=malloc(pCount*sizeof(GWEN_DB_NODE *));
+      assert(orderedProfiles);
+      n=GWEN_DB_GetFirstGroup(dbProfiles);
+      for (i=0; i<pCount && n; i++) {
+        orderedProfiles[i]=n;
+        n=GWEN_DB_GetNextGroup(n);
+      }
+      assert(i==pCount && !n);
+      qsort(orderedProfiles, pCount, sizeof(GWEN_DB_NODE *),
+            AH_Job__SortSepaProfiles);
+      GWEN_DB_UnlinkGroup(orderedProfiles[0]);
+      j->sepaProfile=orderedProfiles[0];
+      free(orderedProfiles);
+    }
+    else {
+      DBG_ERROR(AQHBCI_LOGDOMAIN,
+                "No supported SEPA format found for job \"%s\"", j->name);
+    }
+    GWEN_DB_Group_free(dbProfiles);
+  }
+  else {
+    DBG_ERROR(AQHBCI_LOGDOMAIN,
+              "No SEPA profiles found");
+  }
+
+  return j->sepaProfile;
+}
+
+
+
+
+#include "job_commit.c"
+
+
