@@ -92,6 +92,15 @@ void AH_User_Flags_toDb(GWEN_DB_NODE *db, const char *name,
   if (flags & AH_USER_FLAGS_TAN_OMIT_SMS_ACCOUNT)
     GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT, name,
                          "omitSmsAccount");
+  if (flags & AH_USER_FLAGS_TLS_ONLY_SAFE_CIPHERS)
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT, name,
+                         "tlsOnlySafeCiphers");
+  if (flags & AH_USER_FLAGS_USE_STRICT_SEPA_CHARSET)
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT, name,
+			 "useStrictSepaCharset");
+  if (flags & AH_USER_FLAGS_TLS_IGN_PREMATURE_CLOSE)
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_DEFAULT, name,
+			 "tlsIgnPrematureClose");
 }
 
 
@@ -123,6 +132,12 @@ uint32_t AH_User_Flags_fromDb(GWEN_DB_NODE *db, const char *name) {
       fl|=AH_USER_FLAGS_KEEP_MULTIPLE_BLANKS;
     else if (strcasecmp(s, "omitSmsAccount")==0)
       fl|=AH_USER_FLAGS_TAN_OMIT_SMS_ACCOUNT;
+    else if (strcasecmp(s, "tlsOnlySafeCiphers")==0)
+      fl|=AH_USER_FLAGS_TLS_ONLY_SAFE_CIPHERS;
+    else if (strcasecmp(s, "useStrictSepaCharset")==0)
+      fl|=AH_USER_FLAGS_USE_STRICT_SEPA_CHARSET;
+    else if (strcasecmp(s, "tlsIgnPrematureClose")==0)
+      fl|=AH_USER_FLAGS_TLS_IGN_PREMATURE_CLOSE;
     else {
       DBG_WARN(AQHBCI_LOGDOMAIN, "Unknown user flag \"%s\"", s);
     }
@@ -370,6 +385,14 @@ void AH_User_ReadDb(AB_USER *u, GWEN_DB_NODE *db) {
   /* read some settings */
   ue->maxTransfersPerJob=GWEN_DB_GetIntValue(db, "maxTransfersPerJob", 0, AH_USER_MAX_TRANSFERS_PER_JOB);
   ue->maxDebitNotesPerJob=GWEN_DB_GetIntValue(db, "maxDebitNotesPerJob", 0, AH_USER_MAX_DEBITNOTES_PER_JOB);
+  free(ue->sepaTransferProfile);
+  s=GWEN_DB_GetCharValue(db, "sepaTransferProfile", 0, NULL);
+  if (s) ue->sepaTransferProfile=strdup(s);
+  else ue->sepaTransferProfile=NULL;
+  free(ue->sepaDebitNoteProfile);
+  s=GWEN_DB_GetCharValue(db, "sepaDebitNoteProfile", 0, NULL);
+  if (s) ue->sepaDebitNoteProfile=strdup(s);
+  else ue->sepaDebitNoteProfile=NULL;
 
   free(ue->tanMediumId);
   s=GWEN_DB_GetCharValue(db, "tanMediumId", 0, NULL);
@@ -492,6 +515,12 @@ void AH_User_toDb(AB_USER *u, GWEN_DB_NODE *db) {
   GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
 		      "maxDebitNotesPerJob",
 		      ue->maxDebitNotesPerJob);
+  if (ue->sepaTransferProfile)
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			 "sepaTransferProfile", ue->sepaTransferProfile);
+  if (ue->sepaDebitNoteProfile)
+    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
+			 "sepaDebitNoteProfile", ue->sepaDebitNoteProfile);
   if (ue->tanMediumId)
     GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
 			 "tanMediumId", ue->tanMediumId);
@@ -673,6 +702,97 @@ GWEN_DB_NODE *AH_User_GetUpd(const AB_USER *u){
 
 
 
+GWEN_DB_NODE *AH_User_GetUpdForAccountIdAndSuffix(const AB_USER *u,
+                                                  const char *sAccountNumber,
+                                                  const char *sAccountSuffix) {
+  AH_USER *ue;
+  GWEN_DB_NODE *db;
+  GWEN_BUFFER *tbuf;
+
+  assert(u);
+  ue=GWEN_INHERIT_GETDATA(AB_USER, AH_USER, u);
+  assert(ue);
+
+  db=AH_User_GetUpd(u);
+  if (db==NULL) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "No upd");
+    return NULL;
+  }
+
+  tbuf=GWEN_Buffer_new(0, 64, 0, 1);
+  GWEN_Buffer_AppendString(tbuf, sAccountNumber);
+  GWEN_Buffer_AppendString(tbuf, "-");
+  /* take into account the "Unterkontomerkmal", don't rely solely on account id */
+  if (sAccountSuffix && *sAccountSuffix)
+    GWEN_Buffer_AppendString(tbuf, sAccountSuffix);
+  else
+    GWEN_Buffer_AppendString(tbuf, "none");
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Checking upd for account \"%s\"", GWEN_Buffer_GetStart(tbuf));
+  db=GWEN_DB_GetGroup(db,
+                      GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+                      GWEN_Buffer_GetStart(tbuf));
+  GWEN_Buffer_free(tbuf);
+
+  if (db==NULL) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "Falling back to old storage of UPD for account \"%s\"", sAccountNumber);
+    db=AH_User_GetUpd(u);
+    db=GWEN_DB_GetGroup(db,
+                        GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+                        sAccountNumber);
+  }
+
+  return db;
+}
+
+
+
+GWEN_DB_NODE *AH_User_GetUpdForAccount(const AB_USER *u, const AB_ACCOUNT *acc){
+  GWEN_DB_NODE *db=NULL;
+
+  db=AH_User_GetUpdForAccountUniqueId(u, AB_Account_GetUniqueId(acc));
+  if (db==NULL) {
+    DBG_INFO(AQHBCI_LOGDOMAIN,
+             "Falling back to previous storage of UPD for account \"%u\"",
+             AB_Account_GetUniqueId(acc));
+    db=AH_User_GetUpdForAccountIdAndSuffix(u,
+                                           AB_Account_GetAccountNumber(acc),
+                                           AB_Account_GetSubAccountId(acc));
+  }
+
+  return db;
+}
+
+
+
+GWEN_DB_NODE *AH_User_GetUpdForAccountUniqueId(const AB_USER *u, uint32_t uid) {
+  AH_USER *ue;
+  GWEN_DB_NODE *db;
+  char numbuf[32];
+
+  assert(u);
+  ue=GWEN_INHERIT_GETDATA(AB_USER, AH_USER, u);
+  assert(ue);
+
+  db=AH_User_GetUpd(u);
+  if (db==NULL) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "No upd");
+    return NULL;
+  }
+
+  snprintf(numbuf, sizeof(numbuf)-1, "uaid-%08llx",
+           (unsigned long long int) uid);
+  numbuf[sizeof(numbuf)-1]=0;
+
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Checking upd for \"%s\"", numbuf);
+  db=GWEN_DB_GetGroup(db,
+                      GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+                      numbuf);
+
+  return db;
+}
+
+
+
 void AH_User_SetUpd(AB_USER *u, GWEN_DB_NODE *n){
   AH_USER *ue;
 
@@ -2072,5 +2192,68 @@ const char *AH_User_FindSepaDescriptor(AB_USER *u, const char *tmpl) {
 
 
 
+const GWEN_STRINGLIST *AH_User_GetSepaDescriptors(AB_USER *u) {
+  AH_USER *ue;
+
+  assert(u);
+  ue=GWEN_INHERIT_GETDATA(AB_USER, AH_USER, u);
+  assert(ue);
+
+  return ue->sepaDescriptors;
+}
 
 
+
+const char *AH_User_GetSepaTransferProfile(const AB_USER *u) {
+  const AH_USER *ue;
+
+  assert(u);
+  ue=GWEN_INHERIT_GETDATA(AB_USER, AH_USER, u);
+  assert(ue);
+
+  return ue->sepaTransferProfile;
+}
+
+
+
+void AH_User_SetSepaTransferProfile(AB_USER *u, const char *profileName) {
+  AH_USER *ue;
+
+  assert(u);
+  ue=GWEN_INHERIT_GETDATA(AB_USER, AH_USER, u);
+  assert(ue);
+
+  free(ue->sepaTransferProfile);
+  if (profileName)
+    ue->sepaTransferProfile=strdup(profileName);
+  else
+    ue->sepaTransferProfile=NULL;
+}
+
+
+
+const char *AH_User_GetSepaDebitNoteProfile(const AB_USER *u) {
+  const AH_USER *ue;
+
+  assert(u);
+  ue=GWEN_INHERIT_GETDATA(AB_USER, AH_USER, u);
+  assert(ue);
+
+  return ue->sepaDebitNoteProfile;
+}
+
+
+
+void AH_User_SetSepaDebitNoteProfile(AB_USER *u, const char *profileName) {
+  AH_USER *ue;
+
+  assert(u);
+  ue=GWEN_INHERIT_GETDATA(AB_USER, AH_USER, u);
+  assert(ue);
+
+  free(ue->sepaDebitNoteProfile);
+  if (profileName)
+    ue->sepaDebitNoteProfile=strdup(profileName);
+  else
+    ue->sepaDebitNoteProfile=NULL;
+}
